#
# spec file for package ffcall
#
# Copyright (c) 2021 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global somajor	0

# This package uses assembly to do its work.  This is the entire list of
# supported architectures understood by RPM, even those not currently supported
# by Fedora.  RPM hasn't heard about line continuations, hence the mess.
%global ffcall_arches %ix86 x86_64 %alpha %arm aarch64 parisc hppa1.0 hppa1.1 hppa1.2 hppa2.0 ia64 m68k mips mipsel ppc ppc64 ppc64le ppc8260 ppc8560 ppc32dy4 ppciseries ppcpseries riscv64 s390 s390x %sparc sparc64

Name:           ffcall
Version:        2.4
Release:        bp154.1.36
Summary:        Libraries for foreign function call interfaces
# As this package only provides a static library together with the header files
License:        GPL-2.0-or-later
Group:          Development/Libraries/Other
Provides:       %{name}-devel = %version-%release
URL:            https://www.gnu.org/software/libffcall/
#               https://git.savannah.gnu.org/cgit/libffcall.git/snapshot/libffcall-<version>.tar.gz
Source0:        https://ftp.gnu.org/gnu/libffcall/libffcall-%{version}.tar.gz
Source1:        https://ftp.gnu.org/gnu/libffcall/libffcall-%{version}.tar.gz.sig
Source2:        https://savannah.gnu.org/project/memberlist-gpgkeys.php?group=libffcall&download=1#/%{name}.keyring
Provides:       ffcall-devel
Requires:       libffcall%{somajor} = %{version}
ExclusiveArch:  %{ffcall_arches}

%if ! %{defined _rpmmacrodir}
%define _rpmmacrodir %{_sysconfdir}/rpm
%endif

%description
This is a collection of four libraries which can be used to build
foreign function call interfaces in embedded interpreters.  The four
packages are:
 - avcall: calling C functions with variable arguments
 - vacall: C functions accepting variable argument prototypes
 - trampoline: closures as first-class C functions
 - callback: closures with variable arguments as first-class C functions
   (a reentrant combination of vacall and trampoline)

%package -n libffcall%{somajor}
Summary:        Libraries for foreign function call interfaces
Group:          System/Libraries

%description -n libffcall%{somajor}
This is a collection of four libraries which can be used to build
foreign function call interfaces in embedded interpreters.  The four
packages are:
 - avcall: calling C functions with variable arguments
 - vacall: C functions accepting variable argument prototypes
 - trampoline: closures as first-class C functions
 - callback: closures with variable arguments as first-class C functions
   (a reentrant combination of vacall and trampoline)

%prep
%setup -q -n libffcall-%{version}
if ! test -e /usr/include/asm/cachectl.h
then
    # FIX-OPENSUSE -- Remove this if there is no <asm/cachectl.h>
    for c in callback/trampoline_r/trampoline.c trampoline/trampoline.c
    do
	echo 'No <asm/cachectl.h> on this architecture %arch'
	sed -ri '/^#ifdef linux/,/^#else/{ \@#include <asm/cachectl.h>@d }' $c
    done
fi

# Remove prebuilt object files
find . -name \*.o | xargs rm -f

%build
%global _lto_cflags %{?_lto_cflags} -ffat-lto-objects
export CFLAGS="%{optflags} -g -fPIC -fno-strict-aliasing"
export STRIP=true STRIPPROG=true
%configure --disable-rpath --with-gnu-ld --enable-thread=posix
if test -e /.build.log
then
    grep -E '^#define (CODE|HAVE)' config.h
    sed -rn '/checking whether code in malloc\(\)ed memory is executable/,/result: /p' config.log
fi
make # %{?_smp_mflags}

%install
mkdir -p %{buildroot}%{_libdir}
mkdir -p %{buildroot}%{_mandir}
mkdir -p %{buildroot}%{_defaultdocdir}/%{name}
export STRIP=true STRIPPROG=true
make install DESTDIR=%{buildroot} STRIP=true STRIPPROG=true
rm -fr %{buildroot}%{_datadir}/html
rm -f %{buildroot}%{_libdir}/*.la
if test %debug = yes ; then
    install config.log %{buildroot}%{_defaultdocdir}/%{name}
    echo %{_defaultdocdir}/%{name}/config.log > list
else
    > list
fi
cd %{buildroot}%{_mandir}/man3

# Advertise supported architectures
mkdir -p %{buildroot}%{_rpmmacrodir}
cat > %{buildroot}%{_rpmmacrodir}/macros.%{name} << EOF
# arches that ffcall supports
%%ffcall_arches %{ffcall_arches}
EOF

# Fix man pages with overly generic names (bz 800360)
for page in *; do
  mv $page %{name}-$page
done

%post -n libffcall%{somajor} -p /sbin/ldconfig
%postun -n libffcall%{somajor} -p /sbin/ldconfig

%files -f list
%defattr(-,root,root,-)
%if 0%{?sle_version} > 120200 || 0%{?suse_version} > 1320
%license COPYING
%else
%doc COPYING
%endif
%doc README NEWS
%doc avcall/avcall.html
%doc callback/callback.html
%doc callback/trampoline_r/trampoline_r.html
%doc trampoline/trampoline.html
%doc vacall/vacall.html
%{_libdir}/*.a
%{_libdir}/*.so
%{_includedir}/*
%doc %{_mandir}/man*/*
%{_rpmmacrodir}/macros.%{name}

%files -n libffcall%{somajor}
%defattr(-,root,root,-)
%{_libdir}/*.so.*

%changelog
* Mon Nov  1 2021 Ferdinand Thiessen <rpm@fthiessen.de>
- Update to version 2.4
  * Fixed a couple of multithread-safety bugs in the trampoline and
    callback packages.
- Remove unused ffcall-trampoline.patch
* Thu May  6 2021 Dr. Werner Fink <werner@suse.de>
- Move /etc/rpm/macros.ffcall to /usr/lib/rpm/macros.d/ (boo#1185666)
* Mon Oct 14 2019 Dr. Werner Fink <werner@suse.de>
- Avoid recursion trouble in spec file cause by undefined _lto_cflags
* Thu Sep 12 2019 Andreas Schwab <schwab@suse.de>
- Add riscv64 to ffcall_arches
* Tue Sep 10 2019 Christophe Giboudeaux <christophe@krop.fr>
- Use -ffat-lto-objects when building static libraries.
* Mon Aug 26 2019 Dr. Werner Fink <werner@suse.de>
- Update to (lib)fcall-2.2 release
  * Added support for the following platforms:
    (Previously, a build on these platforms failed.)
  - armv6 (Raspberry Pi): Linux.
  - hppa: Linux 32-bit.
  - riscv32: Linux with ilp32d ABI.
  - riscv64: Linux with lp64d ABI.
  * The build for the mips 32-bit ABI is now compatible with toolchains for
    the 'fpxx' ABI variant.  The downside is that the mips1 architecture is
    no longer supported.
  * Fixed a bug regarding passing of more than 8 arguments on the following
    platforms:
  - arm64: Linux 64-bit.
  * Fixed a stack corruption bug on the following platforms:
  - hppa: HP-UX 32-bit.
* Sun Aug 25 2019 Andreas Stieger <andreas.stieger@gmx.de>
- add upstream signing keyring and verify source signature
* Wed Jul 10 2019 Dr. Werner Fink <werner@suse.de>
- Fix dependencies to get clisp build again
* Tue Feb 20 2018 werner@suse.de
- Correct order of %%file and %%defattr
* Tue Feb 20 2018 werner@suse.de
- Allow debuginfo
* Mon Feb 19 2018 werner@suse.de
- Update to (lib)fcall-2.1 release
  * Added support for the following platforms:
    (Previously, a build on these platforms failed.)
  - x86_64: Solaris 11.3.
  - OpenBSD 6.1.
  - HardenedBSD 10 and 11.
  * Fixed a bug regarding passing of pointers on the following platforms:
  - x86_64: Linux with x32 ABI: CC="gcc -mx32".
  * Fixed a crash in trampoline on the following platforms:
- Update the patch ffcall-trampoline.patch even if not applied
* Mon Sep 11 2017 werner@suse.de
- Update to (lib)fcall-2.0 release
  * The package now installs a library libffcall.{a,so}. It contains the
    'avcall' and 'callback' packages. The libraries libavcall.{a,so} and
    libcallback.{a,so} are still installed as well, but are deprecated.
  * The installed libraries are now installed as shared libraries by default
    (except for libvacall, which is still a static library only).
  * The installed shared libraries are now properly versioned. This means that
    when installing with --enable-shared, upgrading to a newer version of
    libffcall will not break existing binaries.
  * The installed include files are now platform independent. This means that
    you can now install libffcall for different ABIs on the same system, using
    the same --prefix option but different --exec-prefix options for each ABI.
  * API changes in <trampoline.h>:
  - The second argument of alloc_trampoline() is now a 'void**', rather
    than a 'void*'.
  - The return value of trampoline_variable() is now a 'void**', rather
    than a 'void*'.
  - The argument of trampoline_address(), trampoline_variable(),
    trampoline_data() is now a function pointer instead of a 'void*'.
  * Fixed a bug regarding floating-point arguments followed by
    non-floating-point arguments on the following platforms:
  - arm: Linux 32-bit, with hardware floats.
  - powerpc: Linux 32-bit.
  - s390: Linux.
  * Fixed a bug regarding structure returns on the following platforms:
  - i386: FreeBSD.
  - i386: MinGW.
  - mips: old 32-bit ABI (Linux, IRIX).
  * Added support for the following platforms:
    (Previously, a build on these platforms failed.)
  - i386: MSVC 14.
  - x86_64: Cygwin.
  - x86_64: MinGW.
  - x86_64: MSVC 14.
  - hppa64: HP-UX 11.
  - m68k: Linux.
  * Verified support for the following platforms:
    (A build on these platforms worked and still works.)
  - i386: FreeBSD, NetBSD, OpenBSD, DragonFly BSD.
  - i386: Hurd.
  - x86_64: FreeBSD, NetBSD, OpenBSD.
- New package libffcall0 to hold the new shared libraries
  mainly libffcall.so.0.0.0
* Wed Jun 28 2017 werner@suse.de
- Accordingly to Bruno Haible the define -DMAP_VARIABLE=MAP_PRIVATE
  should become removed
* Mon Jun 26 2017 werner@suse.de
- Add some more debug code for build architecture
* Mon Jun 26 2017 werner@suse.de
- Update to (lib)fcall-1.13 release
  * The license has been changed from GPLv2 to GPLv2+.
  * Added support for the following platforms:
    (Previously, a build on these platforms failed.)
  - x86_64: Mac OS X 64-bit.
  - x86_64: Solaris 64-bit.
  - x86_64: Linux with x32 ABI: CC="gcc -mx32".
  - arm: Linux 32-bit, without hardware floats.
  - arm64: Linux 64-bit.
  - s390x: Linux 64-bit.
  - powerpc: AIX 64-bit.
  - mips: IRIX 6.5 with CC="cc -32".
  - sparc: Solaris 64-bit.
  * Fixed support for the following platforms:
    (Previously, a build on these platforms appeared to succeed but was buggy.)
  - x86_64: Linux.
  - arm: Linux 32-bit, with hardware floats.
  - powerpc: Linux 64-bit.
  - mips: Linux with CC="gcc -mabi=32".
  - mips: Linux with CC="gcc -mabi=n32".
  - mips: Linux with CC="gcc -mabi=64".
  - mips: IRIX 6.5 with CC="gcc -mabi=n32".
  - s390: Linux.
  - sparc: Linux 64-bit.
  - ia64: Linux.
  - hppa: HP-UX 32-bit.
  * Verified support for the following platforms:
    (A build on these platforms worked and still works.)
  - i386: Linux, Solaris, Mac OS X.
  - powerpc: Linux 32-bit.
  - powerpc: AIX 32-bit.
  - powerpc: MacOS X.
  - mips: IRIX 6.5 with CC="cc -n32".
  - sparc: Solaris 32-bit.
  - sparc: Linux 32-bit: CC="gcc -m32".
  - alpha: Linux.
  * Support for a security feature: On Linux and FreeBSD platforms, linking with
    the libffcall libraries no longer causes the stack to become executable.
- Remove the patch ffcall-asm_cachectl.patch
- Update the patch ffcall-trampoline.patch
* Sat Feb 13 2016 schwab@linux-m68k.org
- Update to ffcall-1.12 release
  * Added ppc64le to the list of supported architectures.
  * Header now define LIBFFCALL_VERSION (to 0x010B).
  * Better support for IA64 on Linux (kernel 2.6.16+ and gcc 4.1.0+).
  * Added ARM support from Jonathan Olson (debian 1.10-2).
  * Added MIPSel support from Thiemo Seufer (debian 1.10-2).
  * Added ARMel support from Max Lapan.
* Wed Feb 12 2014 ro@suse.de
- remove s390x from list of architectures (no asm code available)
* Tue May 14 2013 werner@suse.de
- Add missing BuildRoot tag
* Wed May  8 2013 werner@suse.de
- Adopt the ffcall package as provided by David Lanzendörfer, his
  version is mainly based on the Fedora ffcall package (SR#172680):
  + Wed Mar 27 2013 Jerry James <loganjerry@gmail.com> - 1.10-10.20120424cvs
  - Update to CVS 20120424
  - List all architectures supported by this package (bz 925335)
  - Rename man pages to avoid conflicts (bz 800360)
  - Add Provides: ffcall-static
  + Wed Feb 13 2013 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 1.10-9.20100903cvs
  - Rebuilt for https://fedoraproject.org/wiki/Fedora_19_Mass_Rebuild
  + Thu Jul 19 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 1.10-8.20100903cvs
  - Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild
  + Tue Jan 10 2012 Jerry James <loganjerry@gmail.com> - 1.10-7.20100903cvs
  - Clean out prebuilt object files
  - Add trampoline patch to force use of mmap() to get executable memory
  + Mon Jan  9 2012 Jerry James <loganjerry@gmail.com> - 1.10-6.20100903cvs
  - Update to CVS 20100903
  - Minor spec file cleanups
  + Tue Feb 08 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 1.10-5.20080704cvs.1
  - Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild
  + Fri Jul 24 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 1.10-4.20080704cvs.1
  - Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild
  + Tue Feb 24 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 1.10-3.20080704cvs.1
  - Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild
  + Wed Dec 10 2008 Jochen Schmitt <Jochen herr-schmitt de> - 1.10-2.20080704cvs.1
  - Fix -FPIC issue (BZ #475112)
  + Fri Jul  4 2008 Gerard Milmeister <gemi@bluewin.ch> - 1.10-2.20080704cvs
  - update to cvs 20080704
  - support for ppc64
  + Mon Feb 25 2008 Gerard Milmeister <gemi@bluewin.ch> - 1.10-1
  - first Fedora release
- Correct group tag, provide ffcall-devel not ffcall-static
- Declare patch ffcall-trampoline.patch
- Add patch ffcall-asm_cachectl.patch
- Correct license string
- Declare configuration file
- Declare manual pages as documentations
